/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-core Project
//
// GeneCanvas.java
// Since: Jan 17, 2008
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/graphics/GenomeCanvas.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.graphics;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;

/**
 * A canvas class for drawing genes
 * 
 * @author leo
 * 
 */
public class GenomeCanvas
{
    private final GenomeWindow window;
    private final BufferedImage image;
    private final Graphics2D g;
    private final int canvasWidth;
    private final int canvasHeight;

    private int yOffset = 0;

    public GenomeCanvas(int width, int height, GenomeWindow window)
    {
        this.window = window;
        canvasWidth = width;
        canvasHeight = height;

        image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        g = (Graphics2D) image.createGraphics();
    }

    public void drawTag(long startIndexOnGenome, long endIndexOnGenome)
    {
        drawTag(startIndexOnGenome, endIndexOnGenome, Color.GRAY);
    }

    public void drawTag(long startIndexOnGenome, long endIndexOnGenome, Color color)
    {
        int start = window.getXPosOnWindow(startIndexOnGenome, canvasWidth);
        int end = window.getXPosOnWindow(endIndexOnGenome, canvasWidth);

        int width = end - start;
        if (width <= 0)
            width = 1;
        final int geneHeight = 5;
        drawRect(start, yOffset, width, geneHeight - 1, color);
        yOffset += geneHeight;
        if (yOffset > canvasHeight)
            yOffset = 0;
    }

    public void drawRect(long startIndexOnGenome, long endIndexOnGenome, int yOffset, int height, Color color)
    {
        int start = window.getXPosOnWindow(startIndexOnGenome, canvasWidth);
        int end = window.getXPosOnWindow(endIndexOnGenome, canvasWidth);

        int width = end - start;
        if (width <= 0)
            width = 1;

        drawRect(start, yOffset, width, height, color);
    }

    public void drawRect(int x, int y, int width, int height, Color color)
    {
        Rectangle2D rect = new Rectangle2D.Double(x, y, width, height);
        g.setColor(color);
        g.fill(rect);
    }

    /**
     * @param text
     * @param startIndexOnGenome
     *            font range start
     * @param endIndexOnGenome
     *            font range end
     * @param fontSize
     * @param color
     */
    public void drawText(String text, long startIndexOnGenome, long endIndexOnGenome, int yOffset, int fontSize,
            Color color)
    {
        int start = window.getXPosOnWindow(startIndexOnGenome, canvasWidth);
        int end = window.getXPosOnWindow(endIndexOnGenome, canvasWidth);
        int width = (start < end) ? end - start : 1;

        g.setFont(new Font("SansSerif", Font.PLAIN, fontSize));
        FontMetrics fontMetrics = g.getFontMetrics();
        int fontHeight = fontMetrics.getHeight();
        int fontWidth = fontMetrics.stringWidth(text);

        g.setColor(color);
        int xOffset = (width - fontWidth) / 2;
        g.drawString(text, start + xOffset, yOffset);
    }

    public void outputImage(HttpServletResponse response, final String imageType) throws IOException
    {
        response.setContentType("image/" + imageType);
        ImageIO.write(image, imageType, response.getOutputStream());
    }

}
