/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// UTGB Gallery Project
//
// TrackRangeSelector.java
// Since: Apr 3, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/track/TrackRangeSelector.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.track;


import org.utgenome.gwt.utgb.client.ui.AbsoluteFocusPanel;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.MouseListenerAdapter;
import com.google.gwt.user.client.ui.Widget;

/**
 * {@link TrackRangeSelector} supports range selection on a displayed track.
 * In order to use {@link TrackRangeSelector}, your track must implement the {@link RangeSelectable} interface. 
 * 
 * @author leo
 *
 */
public class TrackRangeSelector extends MouseListenerAdapter
{
    boolean isInRangeSelectMode = false;
    HTML rangeIndicator = new HTML();

    RangeSelectable track;
    AbsoluteFocusPanel trackPanel;
    int x1 = -1;
    int x2 = -1;
    int windowWidth = 800; 
    
    public TrackRangeSelector(RangeSelectable track)
    {
        this.track = track;
        this.trackPanel = track.getAbsoluteFocusPanel();
        rangeIndicator.setStyleName("track-range");
        rangeIndicator.setHeight("10px");
        trackPanel.addMouseListener(this);
    }
    
    public void setWindowWidth(int windowWidth)
    {
        this.windowWidth = windowWidth;
    }
    
    /**
     * An event handler when a mouse down event is reported from a child widget of the trackPanel. 
     * @param x
     * @param y
     */
    public void onMouseDownFromChild(int x, int y)
    {
        int xOnWindow = x - trackPanel.getFocusPanel().getAbsoluteLeft();
        if(xOnWindow < 0)
            xOnWindow = 0;
        onMouseDown(trackPanel.getFocusPanel(), xOnWindow, y);
    }

    public void onMouseDown(Widget sender, int x, int y)
    {
        if(!isInRangeSelectMode)
        {
            DOM.setCapture(sender.getElement());
            x1 = x;
            rangeIndicator.setWidth("1px");
            trackPanel.add(rangeIndicator, x1, 0);
        }
        else
        {
            DOM.releaseCapture(sender.getElement());

            x2 = (x < windowWidth) ? x : windowWidth;
            trackPanel.remove(rangeIndicator);
            if(x1 >= 0 && x2 >= 0)
                track.onRangeSelect(x1, x2);
        }
        
        isInRangeSelectMode = !isInRangeSelectMode;
    }

    public void onMouseMove(Widget sender, int x, int y)
    {
        if(!isInRangeSelectMode)
            return;

        int width = x - x1;
        if(width <= 0)
            width = 1;
        if(width >= windowWidth)
            width = windowWidth;
        rangeIndicator.setWidth(width + "px");
    }
    
    
}




